@@ -42,10 +42,10 @@ def array_to_binary(ar, obj=None, force_contiguous=True):
         ar = ar.astype(np.int32)
     if force_contiguous and not ar.flags["C_CONTIGUOUS"]:  # make sure it's contiguous
         ar = np.ascontiguousarray(ar)
-    return {'buffer': memoryview(ar), 'dtype': str(ar.dtype), 'shape': ar.shape}
+    return {'view': memoryview(ar), 'dtype': str(ar.dtype), 'shape': ar.shape}
 
 def binary_to_array(value, obj=None):
-    return np.frombuffer(value['data'], dtype=value['dtype']).reshape(value['shape'])
+    return np.frombuffer(value['view'], dtype=value['dtype']).reshape(value['shape'])
 
 ndarray_serialization = dict(to_json=array_to_binary, from_json=binary_to_array)
 
@@ -77,6 +77,12 @@ class JupyterScatter(widgets.DOMWidget):
     camera_rotation = Float(1).tag(sync=True)
     camera_view = List(None, allow_none=True).tag(sync=True)
 
+    # Zoom properties
+    zoom_to = Array(default_value=None, allow_none=True).tag(sync=True, **ndarray_serialization)
+    zoom_animation = Int(1000).tag(sync=True)
+    zoom_padding = Float(0.333).tag(sync=True)
+    zoom_on_selection = Bool(False).tag(sync=True)
+
     # Interaction properties
     mouse_mode = Enum(['panZoom', 'lasso', 'rotate'], default_value='panZoom').tag(sync=True)
     lasso_initiator = Bool().tag(sync=True)
